==> contrast_agent-9.2.0.tar.gz: contrast_agent-9.2.0/src/contrast/reporting/teamserver_messages/_traces.py <==
# Copyright © 2024 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
from contrast_fireball import ObservedRoute
import requests

from .base_ts_message import BaseTsAppMessage
from contrast.utils.decorators import fail_loudly
from contrast.utils.timer import now_ms
from contrast_vendor import structlog as logging

logger = logging.getLogger("contrast")


class _Traces(BaseTsAppMessage):
    """This class should only be instantiated by Preflight's response handler"""

    def __init__(self, finding, reportable_request):
        super().__init__()

        self.headers.update({"Report-Hash": finding.preflight.split(",")[-1]})

        self.body = self._build_body(finding, reportable_request)

    @property
    def name(self) -> str:
        return "traces"

    @property
    def path(self) -> str:
        return "traces"

    @property
    def request_method(self):
        return requests.Session.put

    @fail_loudly("Failed to process Traces response")
    def process_response(self, response, reporting_client):
        self.process_response_code(response, reporting_client)

    def _route_to_json(self, route: ObservedRoute):
        return {
            # "The number of times this route was observed; must be more than 0"
            "count": 1,
            "observations": [{"url": route.url, "verb": route.verb}],
            "signature": route.signature,
        }

    def _build_body(self, finding, reportable_request):
        base_body = {
            "created": (
                finding.events[-1].timestamp_ms if len(finding.events) > 0 else now_ms()
            ),
            "events": [event.to_json() for event in finding.events],
            "properties": finding.properties,
            "routes": [self._route_to_json(route) for route in finding.routes],
            "ruleId": finding.rule_id,
            "tags": self.settings.config.assess_tags,
            "version": finding.version,
        }
        if session_id := self.settings.config.session_id:
            base_body["session_id"] = session_id
        if reportable_request is not None:
            base_body["request"] = reportable_request

        return base_body

==> contrast_agent-9.4.0.tar.gz: contrast_agent-9.4.0/src/contrast/reporting/teamserver_messages/_traces.py <==
# Copyright © 2024 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
from dataclasses import asdict
from typing import Any, Dict, List, Tuple
import requests

from .base_ts_message import BaseTsAppMessage
from contrast.utils.decorators import fail_loudly
from contrast_vendor import structlog as logging

logger = logging.getLogger("contrast")


class _Traces(BaseTsAppMessage):
    """This class should only be instantiated by Preflight's response handler"""

    def __init__(self, finding, reportable_request):
        super().__init__()

        self.headers.update({"Report-Hash": str(finding.hash)})

        self.body = self._build_body(finding, reportable_request)

    @property
    def name(self) -> str:
        return "traces"

    @property
    def path(self) -> str:
        return "traces"

    @property
    def request_method(self):
        return requests.Session.put

    @fail_loudly("Failed to process Traces response")
    def process_response(self, response, reporting_client):
        self.process_response_code(response, reporting_client)

    def _build_body(self, finding, reportable_request):
        base_body = {
            "created": finding.created,
            "events": [as_camel_dict(event) for event in finding.events],
            "properties": finding.properties,
            "routes": [as_camel_dict(route) for route in finding.routes],
            "ruleId": finding.rule_id,
            "tags": finding.tags,
            "version": finding.version,
        }
        if session_id := self.settings.config.session_id:
            base_body["session_id"] = session_id
        if reportable_request is not None:
            base_body["request"] = reportable_request

        return base_body


def as_camel_dict(data):
    return asdict(data, dict_factory=lower_camel_case_keyed_dict_factory)


def lower_camel_case_keyed_dict_factory(kvs: List[Tuple[str, Any]]) -> Dict[str, Any]:
    return {lower_snake_to_camel(k): v for k, v in kvs if v is not None}


def lower_snake_to_camel(snake_str: str) -> str:
    return "".join(
        word if i == 0 else word.capitalize()
        for i, word in enumerate(snake_str.split("_"))
    )
